import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;

class BusinessModel extends Observable implements Runnable {

    private boolean busy_;
    public boolean busy() { return busy_ ; }
    public void busy(boolean aBoolean) { busy_ = aBoolean ; }

    private boolean done_;
    public boolean done() { return done_ ; }
    public void done(boolean aBoolean) { done_ = aBoolean ; }

    private Thread thread_;
    public Thread thread() { return thread_ ; }
    public void thread(Thread aThread) { thread_ = aThread ; }

    public BusinessModel() {
        this.initialize();
    }

    private void initialize() {
        this.busy(false);
        this.done(false);
        this.thread(new Thread(this));
        this.thread().start();
    }

    private void publicize(EventCommand aCommand) {
        this.setChanged();
        this.notifyObservers(aCommand);
    }

    public void run() {

        while (!(this.done())) {

            /*
                A random number generator determines
                which EventCommand will be publicized.

                The EventCommands are Singletons for
                convenience. aCommand will either be
                an instance of BeepOnce, BeepTwice, or
                BeepThrice.
            */

            EventCommand aCommand = null;

            double r = Math.random();

            if      (r < 0.33) aCommand = new BeepOnce();
            else if (r < 0.66) aCommand = new BeepTwice();
            else if (r < 1.00) aCommand = new BeepThrice();

            this.publicize(aCommand);

            try {
                Thread.sleep(3000);
            }
            catch (InterruptedException e) {
                System.out.println("uh..oh....");
            }

        }

        this.busy(false);
    }
}
