import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.image.*;
import java.awt.image.MemoryImageSource;

class GraphTestView extends View implements ActionListener,
                                          WindowListener,
                                          Observer {

    private GraphTest graphTest_;
    public GraphTest graphTest() { return graphTest_; }
    public void graphTest(GraphTest aGT) { graphTest_ = aGT; }

    private GraphView graphViewA_;
    public GraphView graphViewA(){ return graphViewA_; }
    public void graphViewA(GraphView aGV) { graphViewA_ = aGV; }

    private GraphView graphViewB_;
    public GraphView graphViewB(){ return graphViewB_; }
    public void graphViewB(GraphView aGV) { graphViewB_ = aGV; }

    GraphTestView(String windowName){
        super(windowName);
        this.initialize();
    }

    public static void main(String argv[]){
        System.out.println(" starting ");
        GraphTestView theGraphTestView =
                          new GraphTestView("Graph Tester");
        theGraphTestView.show();
    }

    private void initialize(){

        this.addWindowListener(this);

        MenuBar aMenuBar = new MenuBar();

        Menu aMenu = new Menu("File");

        MenuItem aMenuItem = new MenuItem("Open");
        aMenuItem.addActionListener(this);
        aMenuItem.setActionCommand("OPEN");
        aMenu.add(aMenuItem);

        aMenuItem = new MenuItem("Exit");
        aMenuItem.addActionListener(this);
        aMenuItem.setActionCommand("EXIT");
        aMenu.add(aMenuItem);

        aMenuBar.add(aMenu);

        this.setMenuBar(aMenuBar);

        // get the canvas width and height from the config file
        int widthC  = new Integer(Configuration.instance().valueOf("CANVASWIDTH")).intValue();
        int heightC = new Integer(Configuration.instance().valueOf("CANVASHEIGHT")).intValue();


        this.graphTest(new GraphTest());

        // now connect the graphs

        Graph theGraphA = this.graphTest().analyzerA().graph();
        Graph theGraphB = this.graphTest().analyzerB().graph();

        GraphView aGVA = new GraphView(widthC, heightC, theGraphA);
        aGVA.setSize(widthC,heightC);
        aGVA.setLocation(60,60);
        this.add(aGVA);
        this.graphViewA(aGVA);

        GraphView aGVB = new GraphView(widthC, heightC, theGraphB);
        aGVB.setSize(widthC,heightC);
        aGVB.setLocation(widthC+30,60);
        this.add(aGVB);
        this.graphViewB(aGVB);

        this.hookUpObservers();

        this.resize(2*(widthC+50),heightC+100);
    }

    public void actionPerformed(ActionEvent e){

        if(e.getActionCommand() == "OPEN"){
                this.open();
                this.repaint();
        }

        if(e.getActionCommand() == "EXIT"){
            this.dispose();
            System.exit(0);
        }
    }

    public void hookUpObservers(){
        this.graphTest().addObserver(this);
    }

    public void open() {
        this.graphViewA().initializeGraphics();
        this.graphViewB().initializeGraphics();
        this.graphTest().startTheEngine();
    }

    public void update(Graphics g) { paint(g); }

    public void update(Observable obs, Object o) {
        this.repaint();
    }

   public boolean handleEvent(Event e) {
      if (e.id == Event.WINDOW_DESTROY)
      {
         this.hide();
         this.dispose();
         System.exit(0);
      }
      return true;
   }

    public void windowActivated(WindowEvent e){}
    public void windowClosed(WindowEvent e){}
    public void windowClosing(WindowEvent e){
        this.dispose();
        System.exit(0);

    }
    public void windowOpened(WindowEvent e){}
    public void windowDeactivated(WindowEvent e){}
    public void windowIconified(WindowEvent e){}
    public void windowDeiconified(WindowEvent e){}


}
