import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;

class ServoControllerView extends OOView
                 implements MouseMotionListener {


    private RemoteServoController remoteServoController_;
    public RemoteServoController remoteServoController() { return remoteServoController_; }
    public void remoteServoController (RemoteServoController aCtlr) {remoteServoController_ = aCtlr; }

    public int lastY_;
    public int lastY(){ return lastY_;}
    public void lastY(int y){lastY_ = y;}

    public int lastX_;
    public int lastX(){ return lastX_;}
    public void lastX(int x){lastX_ = x;}

	/**
	 *  The offset of the box indicating the arm range of movement
	 */
	private int boxXOffset_;
	public int boxXOffset() { return boxXOffset_; }
	public void boxXOffset( int anInt ) { boxXOffset_ = anInt; }

	/**
	 *  The offset of the  same box
	 */
	private int boxYOffset_;
	public int boxYOffset() { return boxYOffset_; }
	public void boxYOffset( int anInt ) { boxYOffset_ = anInt; }

	/**
	 *  The width
	 */
	private int boxWidth_;
	public int boxWidth() { return boxWidth_; }
	public void boxWidth( int anInt ) { boxWidth_ = anInt; }

	/**
	 *  The height
	 */
	private int boxHeight_;
	public int boxHeight() { return boxHeight_; }
	public void boxHeight( int anInt ) { boxHeight_ = anInt; }

    ServoControllerView(){
        super("Servo Controller");
    }
    public static void main(String argv[]){
        ServoControllerView aServoController = new ServoControllerView();
        aServoController.show();
        aServoController.initialize();
    }
    public void initialize(){

        this.lastX(0);
        this.lastY(0);

        // RMI lookup
        this.findRemoteServoController();

        Label aLabel = new Label("Packets");
        aLabel.setSize(80,20);
        aLabel.setLocation(10,30);
        this.addWidget("listLabel",aLabel);

// box stuff
        this.boxWidth(100);
        this.boxHeight(140);
        this.boxXOffset ((300-100) / 2);
        this.boxYOffset ((300-140) / 2);

        this.addMouseMotionListener(this);
        this.setSize(300,300);
    }

    public void findRemoteServoController() {

        RemoteServoController theRSC = null;

        try {
            System.setSecurityManager(new RMISecurityManager());
            String url = "rmi://allosaurus/RemoteServoController";
            theRSC = (RemoteServoController) Naming.lookup(url);
        }
        catch (RemoteException e) {System.out.println("RMI-Error, Creation Difficulty");}
        catch (Exception e)  {System.out.println("RMI-Error, Unknown difficulty");}

        this.remoteServoController(theRSC); // all was OK set the RSC
    }

    public void scaleMouseData(int x, int y){

        int finalX, finalY;

        finalX = x - boxXOffset() + 50; // ServoController.MINX;
        finalY = y - boxYOffset() + 80; //ServoController.MINY;

        finalX = Math.min(150, finalX);
        finalX = Math.max(50,  finalX);
        finalY = Math.min(220, finalY);
        finalY = Math.max(80,  finalY);


        ((Label)this.getWidget("listLabel")).setText(
                String.valueOf(finalX) + " " +
                String.valueOf(finalY));

        try {
            this.remoteServoController().updatePositionTo( finalX, finalY );
        }
        catch (RemoteException e) {System.out.println("RMI - Error at updatePositionTo");}
    }

    public void mouseMoved(MouseEvent e){

        this.lastX(e.getX());
        this.lastY(e.getY());

        this.scaleMouseData( e.getX() , e.getY() );
        this.repaint();
    }

    public void mouseDragged(MouseEvent e){}
    public void testButton(){}

    public void paint(Graphics g) {

        g.setColor(Color.red);

        g.drawLine(1,this.lastY(),255,this.lastY());
        g.drawLine(this.lastX(),1,this.lastX(),255);

        g.setColor (Color.green);

        g.drawRect (this.boxXOffset(), this.boxYOffset(),
                     this.boxWidth(), this.boxHeight());
    }

}
