import java.util.*;

class TrafficLiteController implements Runnable {

	/**
	 */
	private TLCAdapter tlcAdapter_;
	public TLCAdapter tlcAdapter() { return tlcAdapter_; }
	public void tlcAdapter( TLCAdapter aTLCAdapter ) { tlcAdapter_ = aTLCAdapter; }

	/**
	 *  The traffic light controller will cycle through this collection
	 *  of Lane objects to find the best combination.
	 */
	private Hashtable lanes_;
	public Hashtable lanes() { return lanes_; }
	public void lanes( Hashtable aHashtable ) { lanes_ = aHashtable; }

    TrafficLiteController(){
        this.initialize();
    }
    public void addLane(String key, Lane aLane){
        this.lanes().put(key,aLane);
    }
    public Lane getLane(String key){
       // System.out.println("bad key?  " + key);
        return (Lane) this.lanes().get(key);
    }

    private void initialize(){
        this.lanes( new Hashtable() );
        this.createLanes();
        this.initConflicts();
       // this.startHeartBeat();
    }
    public void initConflicts(){

        Enumeration enum = this.lanes().elements();
        while( enum.hasMoreElements() ){
            Lane aLane = (Lane) enum.nextElement();
            aLane.setConflicts();
        }
    }


    public void instructLight(String laneKey , String color){

        if( laneKey.equals("NSLight") ){
            this.tlcAdapter().northThruLite(color);
        }
        if( laneKey.equals("NLLight") ){
            this.tlcAdapter().northLeftLite(color);
        }
        if( laneKey.equals("SSLight") ){
            this.tlcAdapter().southThruLite(color);
        }
        if( laneKey.equals("SLLight") ){
            this.tlcAdapter().southLeftLite(color);
        }
        if( laneKey.equals("ESLight") ){
            this.tlcAdapter().eastThruLite(color);
        }
        if( laneKey.equals("ELLight") ){
            this.tlcAdapter().eastLeftLite(color);
        }
        if( laneKey.equals("WSLight") ){
            this.tlcAdapter().westThruLite(color);
        }
        if( laneKey.equals("WLLight") ){
            this.tlcAdapter().westLeftLite(color);
        }


        if( laneKey.equals("NPedLight1") ){
            this.tlcAdapter().nPedLite(color);
        }
        if( laneKey.equals("NPedLight2") ){
            this.tlcAdapter().nPedLite(color);
        }
        if( laneKey.equals("SPedLight1") ){
            this.tlcAdapter().sPedLite(color);
        }
        if( laneKey.equals("SPedLight2") ){
            this.tlcAdapter().sPedLite(color);
        }
        if( laneKey.equals("EPedLight1") ){
            this.tlcAdapter().ePedLite(color);
        }
        if( laneKey.equals("EPedLight2") ){
            this.tlcAdapter().ePedLite(color);
        }
        if( laneKey.equals("WPedLight1") ){
            this.tlcAdapter().wPedLite(color);
        }
        if( laneKey.equals("WPedLight2") ){
            this.tlcAdapter().wPedLite(color);
        }
    }
    public void createLanes(){
 // System.out.println("createLanes in tlc");
		NorthThruLane aNorthThruLane = new NorthThruLane(this);
		this.addLane("NorthThruLane",aNorthThruLane);

		NorthLeftLane aNorthLeftLane = new NorthLeftLane(this);
		this.addLane("NorthLeftLane",aNorthLeftLane);

		SouthThruLane aSouthThruLane = new SouthThruLane(this);
		this.addLane("SouthThruLane",aSouthThruLane);

		SouthLeftLane aSouthLeftLane = new SouthLeftLane(this);
		this.addLane("SouthLeftLane",aSouthLeftLane);

		EastThruLane aEastThruLane = new EastThruLane(this);
		this.addLane("EastThruLane",aEastThruLane);

		EastLeftLane aEastLeftLane = new EastLeftLane(this);
		this.addLane("EastLeftLane",aEastLeftLane);

		WestThruLane aWestThruLane = new WestThruLane(this);
		this.addLane("WestThruLane",aWestThruLane);

		WestLeftLane aWestLeftLane = new WestLeftLane(this);
		this.addLane("WestLeftLane",aWestLeftLane);

		NorthPedLane aNorthPedLane = new NorthPedLane(this);
		this.addLane("NorthPedLane",aNorthPedLane);

		SouthPedLane aSouthPedLane = new SouthPedLane(this);
		this.addLane("SouthPedLane",aSouthPedLane);

		EastPedLane aEastPedLane = new EastPedLane(this);
		this.addLane("EastPedLane",aEastPedLane);

		WestPedLane aWestPedLane = new WestPedLane(this);
		this.addLane("WestPedLane",aWestPedLane);


    }
    public void startHeartBeat(){
        System.out.println("startHeartBeat in TLC");
        (new Thread(this)).start();
    }
    public void callHeartBeatInLanes(){
        Lane aLane;
        Enumeration theLanes = this.lanes().elements();
        while( theLanes.hasMoreElements() ){
            aLane = (Lane) theLanes.nextElement();
            aLane.heartBeat();
        }
    }
    public void heartBeat(){

      //  System.out.println("heartBeat in TLC");
        Vector requestingLanes = this.selectRequestingLanes();
        Enumeration sortedLanes = (ShellSorter.sort(requestingLanes)).elements();
     //   System.out.println("sorted by time");
        Vector chosenLanes = this.createChosenList( sortedLanes );
     //   System.out.println("created chosen");
        this.instructLanes( chosenLanes );
     //   System.out.println("instructed lanes");
        this.callHeartBeatInLanes();
    }
    public void run(){
       // System.out.println("top of run in TLC");
        while(true){
            this.heartBeat();
            this.delay();
        }
    }
    private void delay(){
        try{ Thread.sleep(500); }
        catch(InterruptedException ie){}
    }
    public Vector selectRequestingLanes(){
        Lane aLane;
        Vector requestingLanes = new Vector();
        Enumeration theLanes = this.lanes().elements();
        while( theLanes.hasMoreElements() ){
            aLane = (Lane) theLanes.nextElement();
            if( aLane.hasGreenRequest() ){
                requestingLanes.addElement(aLane);
               // System.out.println(aLane.greenRequest().time());
            }
        }
        return requestingLanes;
    }

    public Vector createChosenList( Enumeration sortedLanes ){
        // here we need to first add all lanes that are green
        //      and are not finished with their minimum time
        //
        Vector chosenLanes = new Vector();
        Lane aLane;
        Enumeration enum = this.lanes().elements();
        while( enum.hasMoreElements() ){
            aLane = (Lane) enum.nextElement();
            if( aLane.mustKeepFlowing()  || aLane.goingGreen()){
              //  System.out.println("must keep flowing or isGoingGreen " + aLane);
                chosenLanes.addElement(aLane);
            }

        }
        while(sortedLanes.hasMoreElements()){
            aLane = (Lane) sortedLanes.nextElement();
            if( aLane.doesNotConflictWith( chosenLanes ) ){
                chosenLanes.addElement( aLane );
            }
        }
        return chosenLanes;
    }

    public void instructLanes( Vector chosenLanes ){
        Lane aLane;
        Enumeration theLanes = this.lanes().elements();
        while( theLanes.hasMoreElements() ){
            aLane = (Lane) theLanes.nextElement();
            if( chosenLanes.contains( aLane ) ){
                if(aLane.isRed() & !aLane.goingGreen()){
                    aLane.goGreen();
                }
              //  System.out.println("said goGreen to " + aLane);
            }
            else{
                if(aLane.isGreen() ){
                    aLane.goRed();
                }
            }
        }
    }
}
