/*+========================================================================+*/
/*| [FBN ARS     ], Copyright [1999] [FBN Software]  All Rights Reserved.  |*/
/*|------------------------------------------------------------------------|*/
/*| NOTE: This software contains valuable trade secrets and proprietary    |*/
/*| information of [FBN Software] and is protected by law. It may not be   |*/
/*| copied or distributed in any form or medium, disclosed to third        |*/
/*| parties, reverse engineered or used in any manner without prior        |*/
/*| written authorization from [FBN Software].                             |*/
/*|------------------------------------------------------------------------|*/
/*| FILE: View.java                                                        |*/
/*|------------------------------------------------------------------------|*/
/*| MODULE:                                                                |*/
/*|------------------------------------------------------------------------|*/
/*| Who | Date       | Description                                         |*/
/*|------------------------------------------------------------------------|*/
/*|  P+ | Nov 29, 99 | Last Code Generation                                |*/
/*+========================================================================+*/
import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class TravelPlanGeneratorView extends SCView {

	/**
	 *  Generates possible TravelPlans given departure
	 *  and destination points
	 */
	private TravelPlanGenerator travelPlanGenerator_;
	private TravelPlanGenerator travelPlanGenerator() { return travelPlanGenerator_; }
	private void travelPlanGenerator( TravelPlanGenerator aTravelPlanGenerator ) { travelPlanGenerator_ = aTravelPlanGenerator; }

    TravelPlanGeneratorView() {
        super ("Fly By Night Travel Plan Generator");
        this.initialize();
    }

    TravelPlanGeneratorView (FlyByNiteReservationSystem theFBNRS) {
        this();
        this.travelPlanGenerator().reservationSystem(theFBNRS);
        this.initChoices(theFBNRS); // needs to be here so reservationSystem is defined
    }

    private void initialize() {

        // initialize
        this.travelPlanGenerator( new TravelPlanGenerator());
        this.setSize(300, 300);
        this.setLocation(100, 100);
        this.initLabels();
        this.initMenu();

        // setup the subviews
        TravelPlansView aTPSView =
                   new TravelPlansView(this.travelPlanGenerator());
        this.addSubView("TravelPlansView", aTPSView);
        this.travelPlanGenerator().addObserver(aTPSView);
   }

    private void initLabels(){

        Font bigFont = new Font("Helvetica",Font.BOLD,20);
        Font mediumFont = new Font("Helvetica",Font.BOLD,14);
        Font smallFont = new Font("Helvetica",Font.BOLD,10);

        Label aLabel = new Label("Departure");
        aLabel.setFont( mediumFont );
        aLabel.setSize(100,32);
        aLabel.setLocation(33,48);
        this.addWidget("departure",aLabel);

        aLabel = new Label("Destination");
        aLabel.setFont( mediumFont );
        aLabel.setSize(100,32);
        aLabel.setLocation(166,48);
        this.addWidget("destination",aLabel);

    }


    private void initMenu(){
        MenuBar aMenuBar = new MenuBar();

            Menu aMenu = new Menu("File");

                MenuItem aMenuItem = new MenuItem("Exit");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Exit");
                aMenu.add(aMenuItem);

            aMenuBar.add(aMenu);

            aMenu = new Menu("Edit");

                aMenuItem = new MenuItem("Cut");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Cut");
                aMenu.add(aMenuItem);

                aMenuItem = new MenuItem("Copy");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Copy");
                aMenu.add(aMenuItem);

                aMenuItem = new MenuItem("Paste");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Paste");
                aMenu.add(aMenuItem);

            aMenuBar.add(aMenu);

            aMenu = new Menu("Run");

                aMenuItem = new MenuItem("Generator");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("GenerateTravelPlans");
                aMenu.add(aMenuItem);

                /*
                aMenuItem = new MenuItem("URL Rankings");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("URLRankings");
                aMenu.add(aMenuItem);
                */

            aMenuBar.add(aMenu);

            /*
            aMenu = new Menu("Tool");

                aMenuItem = new MenuItem("Generate Travel Plans");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("GenerateTravelPlans");
                aMenu.add(aMenuItem);
            */

                /*
                aMenuItem = new MenuItem("URL Rankings");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("URLRankings");
                aMenu.add(aMenuItem);

            aMenuBar.add(aMenu);
                */


        this.setMenuBar(aMenuBar);

    }

    public void initChoices(FlyByNiteReservationSystem theFBNRS) {

        Choice c1 = new Choice();
        c1.setSize(100,30);
        c1.setLocation(33,96);
        this.addWidget("departureChoice", c1);

        Choice c2 = new Choice();
        c2.setSize(100,32);
        c2.setLocation(166,96);
        this.addWidget("destinationChoice", c2);

        // now fill the airports in

        Enumeration theOC = theFBNRS.airportNames().elements();
        while (theOC.hasMoreElements()) {
            String theAirportName = (String) theOC.nextElement();
            c1.addItem(theAirportName);
            c2.addItem(theAirportName);
        }

        // now the dates

        Choice c3 = new Choice();
        c3.setSize(100,32);
        c3.setLocation(33,160);
        this.addWidget("departureDateChoice", c3);

        Choice c4 = new Choice();
        c4.setSize(100,32);
        c4.setLocation(166,160);
        this.addWidget("destinationDateChoice", c4);

        theOC = theFBNRS.flightLegDays().elements();
        while (theOC.hasMoreElements()) {
            String theDate = (String) theOC.nextElement();
            c3.addItem(theDate);
            c4.addItem(theDate);
        }


        // now the times

        Choice c = new Choice();
        c.addItem("00:00");
        c.addItem("01:00");
        c.addItem("02:00");
        c.addItem("03:00");
        c.addItem("04:00");
        c.addItem("05:00");
        c.addItem("06:00");
        c.addItem("07:00");
        c.addItem("08:00");
        c.addItem("09:00");
        c.addItem("10:00");
        c.addItem("11:00");
        c.addItem("12:00");
        c.addItem("13:00");
        c.addItem("14:00");
        c.addItem("15:00");
        c.addItem("16:00");
        c.addItem("17:00");
        c.addItem("18:00");
        c.addItem("19:00");
        c.addItem("20:00");
        c.addItem("21:00");
        c.addItem("22:00");
        c.addItem("23:00");
        c.setSize(100,32);
        c.setLocation(33,224);
        this.addWidget("departureTimeChoice", c);

        c = new Choice();
        c.addItem("00:00");
        c.addItem("01:00");
        c.addItem("02:00");
        c.addItem("03:00");
        c.addItem("04:00");
        c.addItem("05:00");
        c.addItem("06:00");
        c.addItem("07:00");
        c.addItem("08:00");
        c.addItem("09:00");
        c.addItem("10:00");
        c.addItem("11:00");
        c.addItem("12:00");
        c.addItem("13:00");
        c.addItem("14:00");
        c.addItem("15:00");
        c.addItem("16:00");
        c.addItem("17:00");
        c.addItem("18:00");
        c.addItem("19:00");
        c.addItem("20:00");
        c.addItem("21:00");
        c.addItem("22:00");
        c.addItem("23:00");
        c.setSize(100,32);
        c.setLocation(166,224);
        this.addWidget("destinationTimeChoice", c);
    }


    public void initChoicesOld(){

        Choice c = new Choice();
        c.addItem("LAX");
        c.addItem("TEX");
        c.addItem("ORE");
        c.addItem("NYC");
        c.addItem("OKC");
        c.addItem("MIA");
        c.setSize(100,30);
        c.setLocation(33,96);
        this.addWidget("departureChoice", c);

        c = new Choice();
        c.addItem("LAX");
        c.addItem("TEX");
        c.addItem("ORE");
        c.addItem("NYC");
        c.addItem("OKC");
        c.addItem("MIA");
        c.setSize(100,32);
        c.setLocation(166,96);
        this.addWidget("destinationChoice", c);

        c = new Choice();
        c.addItem("12/06");
        c.addItem("12/07");
        c.addItem("12/08");
        c.addItem("12/09");
        c.setSize(100,32);
        c.setLocation(33,160);
        this.addWidget("departureDateChoice", c);

        c = new Choice();
        c.addItem("12/06");
        c.addItem("12/07");
        c.addItem("12/08");
        c.addItem("12/09");
        c.setSize(100,32);
        c.setLocation(166,160);
        this.addWidget("destinationDateChoice", c);

        c = new Choice();
        c.addItem("00:00");
        c.addItem("01:00");
        c.addItem("02:00");
        c.addItem("03:00");
        c.addItem("04:00");
        c.addItem("05:00");
        c.addItem("06:00");
        c.addItem("07:00");
        c.addItem("08:00");
        c.addItem("09:00");
        c.addItem("10:00");
        c.addItem("11:00");
        c.addItem("12:00");
        c.addItem("13:00");
        c.addItem("14:00");
        c.addItem("15:00");
        c.addItem("16:00");
        c.addItem("17:00");
        c.addItem("18:00");
        c.addItem("19:00");
        c.addItem("20:00");
        c.addItem("21:00");
        c.addItem("22:00");
        c.addItem("23:00");
        c.setSize(100,32);
        c.setLocation(33,224);
        this.addWidget("departureTimeChoice", c);

        c = new Choice();
        c.addItem("00:00");
        c.addItem("01:00");
        c.addItem("02:00");
        c.addItem("03:00");
        c.addItem("04:00");
        c.addItem("05:00");
        c.addItem("06:00");
        c.addItem("07:00");
        c.addItem("08:00");
        c.addItem("09:00");
        c.addItem("10:00");
        c.addItem("11:00");
        c.addItem("12:00");
        c.addItem("13:00");
        c.addItem("14:00");
        c.addItem("15:00");
        c.addItem("16:00");
        c.addItem("17:00");
        c.addItem("18:00");
        c.addItem("19:00");
        c.addItem("20:00");
        c.addItem("21:00");
        c.addItem("22:00");
        c.addItem("23:00");
        c.setSize(100,32);
        c.setLocation(166,224);
        this.addWidget("destinationTimeChoice", c);

    }

    public TravelPlanRequest createTravelPlanRequest() {
        TravelPlanRequest theTPR = new TravelPlanRequest();
        theTPR.departurePoint  (((Choice) this.getWidget("departureChoice"))      .getSelectedItem());
        theTPR.destinationPoint(((Choice) this.getWidget("destinationChoice")).    getSelectedItem());
        theTPR.departureDate   (((Choice) this.getWidget("departureDateChoice")).  getSelectedItem());
        theTPR.destinationDate (((Choice) this.getWidget("destinationDateChoice")).getSelectedItem());
        theTPR.departureTime   (((Choice) this.getWidget("departureTimeChoice")).  getSelectedItem());
        theTPR.destinationTime (((Choice) this.getWidget("destinationTimeChoice")).getSelectedItem());
        return theTPR;
    }

    public void generateTravelPlans(){

        TravelPlansView theTPV = (TravelPlansView) this.getSubView("TravelPlansView");
        ((List)theTPV.getWidget("plans")).removeAll(); // clear the list

        this.travelPlanGenerator().generateTravelPlansFrom(this.createTravelPlanRequest());

        theTPV.show();
        System.out.println("Done generating travel plans");
    }

    public void actionPerformed(ActionEvent e){
        String command = (String) e.getActionCommand();
        if(command == "Exit"){
            this.exit();
            return;
        }
        if(command == "GenerateTravelPlans"){
            this.generateTravelPlans();
            return;
        }

    }


    public void windowClosing(WindowEvent e){
        this.exit();
    }

    private void exit(){
        this.hide();
    }

}

