import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class VendView extends SCView {

	private VendingMachine vendingMachine_;
	public VendingMachine vendingMachine() { return vendingMachine_; }
	public void vendingMachine( VendingMachine aVendingMachine ) { vendingMachine_ = aVendingMachine; }

    private VendView(VendingMachine aVendingMachine) {
        super("Vending Machine");
        this.vendingMachine( aVendingMachine );
        this.initialize();
    }

    public static void main( String args[] ) {
        VendView aVendView = new VendView( new VendingMachine() );
        aVendView.show();
    }


    private void initialize() {

        this.initMenu();
        this.initLabels();
        this.initDispensePanels();
        this.initButtons();
        this.initCustomButtons();
        this.initMiscelaneous();

        this.hookupObservers();

        this.setSize(631,492);
        this.setLocation(20,20);


    }

    private void initDispensePanels(){

        int startingPoint = 10;
        int deltaX = 110;
        int panelCount = 0;

        Enumeration names = this.vendingMachine().dispenserNames().elements();
        while( names.hasMoreElements() ){
            String aBrandName = (String) names.nextElement();
            System.out.println("aBrandName = " + aBrandName);
            DispensePanel dp = new DispensePanel(this);
            dp.setBrand(aBrandName);
            dp.setLocation(startingPoint + deltaX * panelCount,120);
            panelCount ++;
            this.addWidget(aBrandName, dp);
        }
    }
    private void initMenu(){
        MenuBar aMenuBar = new MenuBar();

        Menu aMenu = new Menu("File");
        MenuItem aMenuItem = new MenuItem("Exit");
        aMenuItem.addActionListener(this);
        aMenuItem.setActionCommand("Exit");
        aMenu.add(aMenuItem);
        aMenuBar.add(aMenu);

        this.setMenuBar(aMenuBar);

    }
    private void initLabels(){
        Font bigFont = new Font("Helvetica",Font.BOLD,20);
        Font mediumFont = new Font("Helvetica",Font.BOLD,14);
        Font smallFont = new Font("Helvetica",Font.BOLD,10);

        Label aLabel = new Label("Acme Vending Machine");
        aLabel.setFont( bigFont );
        aLabel.setSize(240,50);
        aLabel.setLocation(50,70);
        this.addWidget("acmeVendingMachineLabel",aLabel);

        aLabel = new Label("Sold Out");
        aLabel.setFont( mediumFont );
        aLabel.setSize(80,30);
        aLabel.setLocation(105,155);
      //  this.addWidget("soldOutLabel",aLabel);

        aLabel = new Label("Exact Change Required");
        aLabel.setFont( mediumFont );
        aLabel.setSize(180,30);
        aLabel.setLocation(340,75);
        this.addWidget("exactChangeRequiredLabel",aLabel);

        aLabel = new Label("Total Deposited");
        aLabel.setFont( mediumFont );
        aLabel.setSize(110,25);
        aLabel.setLocation(390,320);
        this.addWidget("totalDeposited",aLabel);
        /*
        aLabel = new Label("Change");
        aLabel.setFont( mediumFont );
        aLabel.setSize(110,25);
        aLabel.setLocation(390,385);
        this.addWidget("change",aLabel);
        */
        aLabel = new ObserverLabel("$0.00");
        aLabel.setFont( bigFont );
        aLabel.setAlignment(aLabel.LEFT);
        aLabel.setSize(100,25);
        aLabel.setLocation(560,265);
        aLabel.setBackground(new Color(200,200,200));
        this.addWidget("totalDepositedLabel",aLabel);

        aLabel = new ObserverLabel("$0.00");
        aLabel.setFont( bigFont );
        aLabel.setAlignment(aLabel.CENTER);
        aLabel.setSize(100,25);
        aLabel.setLocation(400,385);
        aLabel.setBackground(new Color(200,200,200));
        this.addWidget("changeLabel",aLabel);

    }
    private void initButtons(){
      //  this.addWidget("ddButton",new DispenseDrinkButton(this));
        this.addWidget("crbButton",new CRButton(this));
    }

    private void initCustomButtons(){
        this.addWidget("quarterButton", new QuarterButton(this));
        this.addWidget("dimeButton", new DimeButton(this));
        this.addWidget("nickelButton", new NickelButton(this));

    }
    private void initMiscelaneous(){
       // this.addWidget("drinkDispensedCanvas", new DrinkDispensedCanvas(this));
        this.addWidget("coinSlotCanvas", new CoinSlotCanvas(this));
      //  this.addWidget("soldOutCanvas", new SoldOutCanvas(this));
        this.addWidget("exactChangeCanvas", new ExactChangeCanvas(this));

    }
    public void hookupObservers() {


        // here we hook up each DispenserPanel with a Dispenser
        Enumeration names = this.vendingMachine().dispenserNames().elements();
        while( names.hasMoreElements() ){
            String aBrandName = (String) names.nextElement();
            this.vendingMachine().dispenser(aBrandName).
                   dispenserGate().addObserver(  (Observer)
                        ((DispensePanel) this.getWidget(aBrandName)).
                            getWidget("dispense Canvas"));

            this.vendingMachine().
                dispenser(aBrandName).soldOutSensor().
                    soldOutLite().addObserver(  (Observer)
                        ((DispensePanel) this.getWidget(aBrandName)).
                            getWidget("sold out Canvas"));
        }

        this.vendingMachine().moneySlot().addObserver(
            (CoinSlotCanvas) this.getWidget("coinSlotCanvas"));

     //   this.vendingMachine().dispenser(    ).dispenserGate().addObserver(
     //       (DrinkDispensedCanvas) this.getWidget("drinkDispensedCanvas"));

       // this.vendingMachine().dispenser().soldOutSensor().soldOutLite().addObserver(
       //     (SoldOutCanvas) this.getWidget("soldOutCanvas"));

        this.vendingMachine().moneySlot().coinCounter().addObserver(
            (ObserverLabel) this.getWidget("totalDepositedLabel"));

       this.vendingMachine().coinReturnButton().returnGate().addObserver(
            (ObserverLabel) this.getWidget("changeLabel"));

        this.vendingMachine().moneySlot().coinCounter().changeDispenser().addObserver(
            (ObserverLabel) this.getWidget("changeLabel"));

        this.vendingMachine().moneySlot().coinCounter().changeDispenser().changeSensor().exactChangeLite().addObserver(
            (ExactChangeCanvas) this.getWidget("exactChangeCanvas"));

        //this.vendingMachine().
    }

    public void dispenseDrinkButton(String brand){
        System.out.println("dispenseDrinkButton in VendView");
        this.vendingMachine().dispense(brand);
    }
    public void coinReturnButton(){
        System.out.println("coinReturnButton in VendView");
        this.vendingMachine().coinReturnButton().press();

    }
    public void quarterButton(){
        System.out.println("quarterButton in VendView");
        this.vendingMachine().moneySlot().coinReceived(
            new Coin(25) );
    }
    public void dimeButton(){
        System.out.println("dimeButton in VendView");
        this.vendingMachine().moneySlot().coinReceived(
            new Coin(10) );
    }
    public void nickelButton(){
        System.out.println("nickelButton in VendView");
        this.vendingMachine().moneySlot().coinReceived(
            new Coin(5) );
    }
    public void update(Observable obs, Object o) {
        String theString = (String) o;
        System.out.println(">> " + theString);
    }
    public void actionPerformed(ActionEvent ae){
        String actionCommand = ae.getActionCommand();
        if( actionCommand == "Exit" ){
            this.exit();
        }
    }
    public void windowClosing(WindowEvent we){
        this.exit();
    }
    private void exit(){
        System.out.println("closing VendView");
        this.delay();
        System.exit(0);
    }
    public void delay() {
        try { Thread.sleep(1000); }
        catch (InterruptedException e) { System.out.println ("View.delay() - uh..oh");
        }
    }

}
