import java.lang.*;
import java.util.*;

class Bank extends Observable {

    private String name_;
    public String name() { return name_; }
    public void name(String aName) { name_ = aName; }

    private Hashtable accounts_;
    public Hashtable accounts() { return accounts_; }
    public void accounts(Hashtable accountSet) { accounts_ = accountSet; }

    private Date today_;
    public Date today() { return today_; }
    public void today(Date aDate) { today_ = aDate; }

    private TransactionStore transactionStore_;
    public TransactionStore transactionStore() { return transactionStore_; }
    public void transactionStore(TransactionStore theTS) { transactionStore_ = theTS; }

    private TransactionLog transactionLog_;
    public TransactionLog transactionLog() { return transactionLog_; }
    public void transactionLog(TransactionLog theTL) { transactionLog_ = theTL; }

    private int numberOfTransactions_;
    public int numberOfTransactions() { return numberOfTransactions_; }
    public void numberOfTransactions(int i) { numberOfTransactions_ = i; }

    private double cashFlow_;
    public double cashFlow() { return cashFlow_; }
    public void cashFlow(double i) { cashFlow_ = i; }


    Bank() {
        this.initialize();
    }

    private void initialize() {
        this.accounts(new Hashtable());

        CheckingAccount a = new CheckingAccount();
        a.customerName("Fred");
        a.id("100");
        a.balance(66360.0);
        this.accounts().put(a.id(),a);

        CheckingAccount b = new CheckingAccount();
        b.customerName("Bill");
        b.id("101");
        b.balance(22.5);
        this.accounts().put(b.id(),b);

        this.today(new Date());
        this.name("Bank of Antartica");
        this.transactionStore(new TransactionStore(this));
        this.transactionLog(new TransactionLog());
    }

    public Account getAccount(String accountID){

        if(!(this.accounts().containsKey(accountID))){
            return new Account();
        }
        return (Account) this.accounts().get(accountID);
    }

    public void publicize(String aString) {
        this.setChanged();
        this.notifyObservers(aString);
    }

    public Vector processLog(){
        return this.transactionLog().processLog();
    }

    public void processTransactions() {

        Transaction aTransaction = this.transactionStore().getNext();

        while(aTransaction != null) {
            this.processTransaction(aTransaction);
            aTransaction = this.transactionStore().getNext();

            try{ Thread.sleep(500); }
            catch(InterruptedException ie){
                System.out.println("InterruptedException in Bank processTransactions");
            }
        }

        this.transactionLog().processLog();
    }

    public void processTransaction(Transaction aTransaction) {

        this.transactionLog().addTransaction(aTransaction);
        this.cashFlow( this.cashFlow() + aTransaction.amount() );
        this.numberOfTransactions(this.numberOfTransactions()+1) ;

        aTransaction.updateAccount();

        String output = "AccountName="    + aTransaction.customerName() +
                        "  Amount="       + aTransaction.amount() +
                        "  Date="         + aTransaction.timestamp() +
                        "  Balance="      + aTransaction.account().balance()  ;
        this.publicize(output);


    }

}